import {Injectable} from '@angular/core';
import {HttpClient} from "@angular/common/http";
import {EnvironmentSpecificService} from "../../openaireLibrary/utils/properties/environment-specific.service";
import {Observable} from "rxjs";
import {SourceType} from "../../openaireLibrary/monitor/entities/stakeholder";


@Injectable({
    providedIn: 'root'
})
export class StatisticsService {

  numberSources: Map<string, string[]> = new Map<string, string[]>();
  chartSources: Map<SourceType, string[]> = new Map<SourceType, string[]>();

  constructor(private http:HttpClient, private environmentSpecificService: EnvironmentSpecificService) {
    this.environmentSpecificService.subscribeEnvironment().subscribe(properties => {
      this.numberSources.set('statistics', [properties.statisticsAPIURL]);
      this.numberSources.set('search', [properties.searchAPIURLLAst]);
      this.numberSources.set('metrics', [properties.metricsAPIURL]);
      this.chartSources.set('stats-tool', [properties.statisticsFrameNewAPIURL, "http://marilyn.athenarc.gr:8080/stats-api/", "http://88.197.53.71:8080/stats-api/", "https://stats.madgik.di.uoa.gr/stats-api/"]);
      this.chartSources.set('old', [properties.statisticsFrameAPIURL]);
      this.chartSources.set('metrics', [properties.metricsAPIURL]);
      this.chartSources.set('image', [""]);
    })
  }

    getNumbers(source: string, url: string): Observable<any> {
        return this.http.get<any>(this.numberSources.get(source) + url);
    }

    getChartUrl(source: SourceType, url: string): string {
        return this.chartSources.get(source)[0] + url;
    }

    getChartSource(url: string): SourceType {
        let source: SourceType = 'image';
        this.chartSources.forEach((values, key) => {
          values.forEach((value) => {
            if(value !== '' && url.indexOf(value) !== -1) {
                source = key;
            }
          });
        });
        return source;
    }
}
